/*
 * Function        : Composer driver for SH Mobile
 *
 * Copyright (C) 2011 Renesas Electronics Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
#ifndef _SH_MOBILE_COMPOSER_H
#define _SH_MOBILE_COMPOSER_H

/* common kernel and user */

/******************************/
/* define structure for ioctl */
/******************************/
struct cmp_lay_supportpixfmt {
	int level;
	int pixfmt;
};

struct cmp_lay_open {
	int level;
	int bufferid;
};

struct cmp_lay_close {
	int level;
};

struct cmp_lay_disp {
	int level;
	int on;
};

struct cmp_lay_alpha {
	int           level;
	unsigned char alpha;
};

struct cmp_lay_keycolor {
	int           level;
	unsigned int  keycolor;
};

struct cmp_lay_backcolor {
	int           level;
	unsigned int  backcolor;
};


struct cmp_lay_data_size {
	int           level;
	unsigned int  x;
	unsigned int  y;
};

struct cmp_lay_pixfmt {
	int           level;
	unsigned int  pixfmt;
	unsigned int  colorspace;
};

struct cmp_lay_cropsize {
	int           level;
	unsigned int  x;
	unsigned int  y;
};

struct cmp_lay_croppos {
	int           level;
	unsigned int  x;
	unsigned int  y;
};

struct cmp_lay_pos {
	int           level;
	unsigned int  x;
	unsigned int  y;
};

struct cmp_lay_compose_size {
	int           level;
	unsigned int  x;
	unsigned int  y;
};

struct cmp_viewlay {
	int           level;
	unsigned int  data_x;
	unsigned int  data_y;
	unsigned int  croppos_x;
	unsigned int  croppos_y;
	unsigned int  cropsize_x;
	unsigned int  cropsize_y;
	unsigned int  pos_x;
	unsigned int  pos_y;
	unsigned int  compose_x;
	unsigned int  compose_y;
};

struct cmp_layaddr {
	int           level;
	unsigned char *addr;
	unsigned int  datasize;
	unsigned int  app_id;
	unsigned int  offset;
	unsigned char *addr_c0;
	unsigned int  datasize_c0;
	unsigned int  app_id_c0;
	unsigned int  offset_c0;
	unsigned char *addr_c1;
	unsigned int  datasize_c1;
	unsigned int  app_id_c1;
	unsigned int  offset_c1;
};

/************************/
/* define cmd for ioctl */
/************************/

#define IOC_SH_MOBILE_COMP_MAGIC 'S'

#define CMP_IOC_ISLAYEX \
	_IO(IOC_SH_MOBILE_COMP_MAGIC, 0x00)
#define CMP_IOCG_SUPPORTPIXFMT \
	_IOW(IOC_SH_MOBILE_COMP_MAGIC, 0x01, struct cmp_lay_supportpixfmt)
#define CMP_IOCS_OPEN \
	_IOW(IOC_SH_MOBILE_COMP_MAGIC, 0x02, struct cmp_lay_open)
#define CMP_IOCS_CLOSE \
	_IOW(IOC_SH_MOBILE_COMP_MAGIC, 0x03, struct cmp_lay_close)
#define CMP_IOCS_DISP \
	_IOW(IOC_SH_MOBILE_COMP_MAGIC, 0x04, struct cmp_lay_disp)
#define CMP_IOCS_ALPHA \
	_IOW(IOC_SH_MOBILE_COMP_MAGIC, 0x05, struct cmp_lay_alpha)
#define CMP_IOCG_ALPHA \
	_IOWR(IOC_SH_MOBILE_COMP_MAGIC, 0x06, struct cmp_lay_alpha)
#define CMP_IOCS_KEYCOLOR \
	_IOW(IOC_SH_MOBILE_COMP_MAGIC, 0x07, struct cmp_lay_keycolor)
#define CMP_IOCG_KEYCOLOR \
	_IOWR(IOC_SH_MOBILE_COMP_MAGIC, 0x08, struct cmp_lay_keycolor)
#define CMP_IOCS_LAYSIZE \
	_IOW(IOC_SH_MOBILE_COMP_MAGIC, 0x09, struct cmp_lay_data_size)
#define CMP_IOCG_LAYSIZE \
	_IOWR(IOC_SH_MOBILE_COMP_MAGIC, 0x0a, struct cmp_lay_data_size)
#define CMP_IOCS_PIXFMT \
	_IOW(IOC_SH_MOBILE_COMP_MAGIC, 0x0b, struct cmp_lay_pixfmt)
#define CMP_IOCG_PIXFMT \
	_IOWR(IOC_SH_MOBILE_COMP_MAGIC, 0x0c, struct cmp_lay_pixfmt)
#define CMP_IOCS_CROPSIZE \
	_IOW(IOC_SH_MOBILE_COMP_MAGIC, 0x0d, struct cmp_lay_cropsize)
#define CMP_IOCG_CROPSIZE \
	_IOWR(IOC_SH_MOBILE_COMP_MAGIC, 0x0e, struct cmp_lay_cropsize)
#define CMP_IOCS_CROPPOS \
	_IOW(IOC_SH_MOBILE_COMP_MAGIC, 0x0f, struct cmp_lay_croppos)
#define CMP_IOCG_CROPPOS \
	_IOWR(IOC_SH_MOBILE_COMP_MAGIC, 0x10, struct cmp_lay_croppos)
#define CMP_IOCS_POS \
	_IOW(IOC_SH_MOBILE_COMP_MAGIC, 0x10, struct cmp_lay_pos)
#define CMP_IOCG_POS \
	_IOWR(IOC_SH_MOBILE_COMP_MAGIC, 0x11, struct cmp_lay_pos)
#define CMP_IOCS_SIZE \
	_IOW(IOC_SH_MOBILE_COMP_MAGIC, 0x12, struct cmp_lay_compose_size)
#define CMP_IOCG_SIZE \
	_IOWR(IOC_SH_MOBILE_COMP_MAGIC, 0x13, struct cmp_lay_compose_size)
#define CMP_IOCS_VIEWLAY \
	_IOW(IOC_SH_MOBILE_COMP_MAGIC, 0x14, struct cmp_viewlay)
#define CMP_IOCS_LAYADDR \
	_IOW(IOC_SH_MOBILE_COMP_MAGIC, 0x15, struct cmp_layaddr)
#define CMP_IOCG_LAYADDR \
	_IOWR(IOC_SH_MOBILE_COMP_MAGIC, 0x16, struct cmp_layaddr)
#define CMP_IOCS_BACKCOLOR \
	_IOW(IOC_SH_MOBILE_COMP_MAGIC, 0x17, struct cmp_lay_backcolor)
#define CMP_IOCG_BACKCOLOR \
	_IOWR(IOC_SH_MOBILE_COMP_MAGIC, 0x18, struct cmp_lay_backcolor)
#define CMP_IOC_START \
	_IO(IOC_SH_MOBILE_COMP_MAGIC, 0x19)




/*******************/
/* define constant */
/*******************/
#define CMP_OUT_PLANE  0
#define CMP_MAIN_LAYER 1
#define CMP_CH1_LAYER  2
#define CMP_CH2_LAYER  3
#define CMP_CH3_LAYER  4

#define CMPPIXFMT_ARGB8888  0
#define CMPPIXFMT_RGB888    1
#define CMPPIXFMT_RGB565    2
#define CMPPIXFMT_YUV420SP  3
#define CMPPIXFMT_YUV422SP  4
#define CMPPIXFMT_YUV420PL  5 /* reserved */

#define CMPYUVCOLOR_BT601_COMPRESS  0
#define CMPYUVCOLOR_BT601_FULLSCALE 1
#define CMPYUVCOLOR_BT709_COMPRESS  2
#define CMPYUVCOLOR_BT709_FULLSCALE 3

#define CMPKEYCOLOR_OFF 0xFFFFFFFF

#define CMP_ERR_OK	0
#define CMP_ERR_NG	-1
#define CMP_ERR_NG_BASE	0x1000
#define CMP_ERR_NG_OPEN_OUT	(-(CMP_ERR_NG_BASE+0x000))
#define CMP_ERR_NG_OPEN_MAIN	(-(CMP_ERR_NG_BASE+0x001))
#define CMP_ERR_NG_NORESOURCE	(-(CMP_ERR_NG_BASE+0x002))

#define CMP_ERR_NG_VIEWLAY1	(-(CMP_ERR_NG_BASE+0x081)) /* laysize */
#define CMP_ERR_NG_VIEWLAY2	(-(CMP_ERR_NG_BASE+0x082)) /* cropsize */
#define CMP_ERR_NG_VIEWLAY3	(-(CMP_ERR_NG_BASE+0x083)) /* croppos */
#define CMP_ERR_NG_VIEWLAY4	(-(CMP_ERR_NG_BASE+0x084)) /* size */
#define CMP_ERR_NG_VIEWLAY5	(-(CMP_ERR_NG_BASE+0x085)) /* pos */

#define CMP_ERR_NG_LAYERPOS_L0  (-(CMP_ERR_NG_BASE+0x100)) /*out */
#define CMP_ERR_NG_LAYERPOS_L1  (-(CMP_ERR_NG_BASE+0x101)) /*main*/
#define CMP_ERR_NG_LAYERPOS_L2  (-(CMP_ERR_NG_BASE+0x102)) /*l1  */
#define CMP_ERR_NG_LAYERPOS_L3  (-(CMP_ERR_NG_BASE+0x103)) /*l2  */
#define CMP_ERR_NG_LAYERPOS_L4  (-(CMP_ERR_NG_BASE+0x104)) /*l3  */

#define CMP_ERR_NG_INSUFFICIENT_L0  (-(CMP_ERR_NG_BASE+0x110)) /*out */
#define CMP_ERR_NG_INSUFFICIENT_L1  (-(CMP_ERR_NG_BASE+0x111)) /*main*/
#define CMP_ERR_NG_INSUFFICIENT_L2  (-(CMP_ERR_NG_BASE+0x112)) /*l1  */
#define CMP_ERR_NG_INSUFFICIENT_L3  (-(CMP_ERR_NG_BASE+0x113)) /*l2  */
#define CMP_ERR_NG_INSUFFICIENT_L4  (-(CMP_ERR_NG_BASE+0x114)) /*l3  */

#define CMP_ERR_NG_RESIZE_L0  (-(CMP_ERR_NG_BASE+0x120)) /*out */
#define CMP_ERR_NG_RESIZE_L1  (-(CMP_ERR_NG_BASE+0x121)) /*main*/
#define CMP_ERR_NG_RESIZE_L2  (-(CMP_ERR_NG_BASE+0x122)) /*l1  */
#define CMP_ERR_NG_RESIZE_L3  (-(CMP_ERR_NG_BASE+0x123)) /*l2  */
#define CMP_ERR_NG_RESIZE_L4  (-(CMP_ERR_NG_BASE+0x124)) /*l3  */

/********************/
/* define err code  */

#ifdef __KERNEL__
/* please not define __KERNEL__ when build application. */

#include <linux/spinlock.h>
#include <linux/semaphore.h>
#include <linux/list.h>
#include <linux/workqueue.h>
#include <rtapi/screen_graphics.h>
#include <media/sh_mobile_appmem.h>

struct composer_bufer;
struct appshare_list;
struct composer_fh;

/* for internal use for kernel */
struct bufferlink {
	struct             list_head list;
	wait_queue_head_t  *body;
	struct composer_fh *fh;
};

struct composer_buffer {
	spinlock_t          buf_lock;
	struct semaphore    buf_sem;
	struct bufferlink   buf_link;
	unsigned int        buf_id;

	/* for buffer parameters */
	unsigned int        alpha;
	unsigned int        keycolor;
	unsigned int        backcolor;
	unsigned int        data_x;
	unsigned int        data_y;
	unsigned int        data_fmt;
	unsigned int        col_space;
	unsigned int        crop_size_x;
	unsigned int        crop_size_y;
	unsigned int        crop_pos_x;
	unsigned int        crop_pos_y;
	unsigned int        pos_x;
	unsigned int        pos_y;
	unsigned int        size_x;
	unsigned int        size_y;
	int                 rotate;
	int                 mirror;

	/* for buffer address (Y/RGB) */
	unsigned char       *addr;
	unsigned int        app_id;
	unsigned int        offset;
	unsigned char       *app_addr;
	unsigned int        app_size;

	/* for buffer address (UV) */
	unsigned char       *addr_c0;
	unsigned int        app_id_c0;
	unsigned int        offset_c0;
	unsigned char       *app_addr_c0;
	unsigned int        app_size_c0;

	/* for buffer address (V) */
	unsigned char       *addr_c1;
	unsigned int        app_id_c1;
	unsigned int        offset_c1;
	unsigned char       *app_addr_c1;
	unsigned int        app_size_c1;
};


struct composer_info {
	struct bufferlink wqlink;
	unsigned int      id;
	struct composer_buffer *buffer;
	struct appmem_handle  *appinfo[3];
};

struct composer_grapdata {
	wait_queue_head_t       wait_notify;
	int                     status;

	/* rt-api arguments. */
	void                    *handle;
	screen_grap_new         _new;
	screen_grap_initialize  _ini;
	screen_grap_image_blend _blend;
	screen_grap_delete      _del;
	screen_grap_quit        _quit;

	screen_grap_layer       _in[4];

	/* work: temporally generate information. */
	/* --- for stride --- */
	int          work_linebyte[5];
	int          work_linebyte_c0[5];
	int          work_linebyte_c1[5];
	int          work_pixelsize[5];
	int          work_pixelsize_c0[5];
	int          work_pixelsize_c1[5];
	/* --- for crop and size --- */
	int          work_rect_x[5];
	int          work_rect_y[5];
	int          work_rect_w[5];
	int          work_rect_h[5];
	/* --- for address --- */
	unsigned int work_base[5];
	void         *work_point[5];
};

struct composer_fh {
	struct work_struct       fh_wqtask;
	struct list_head         fh_filelist;
	struct semaphore         fh_sem;
	rwlock_t                 fh_rwlock;
	wait_queue_head_t        fh_wait;
	struct composer_info     fh_info[5];
	int                      fh_status;
	struct composer_buffer   fh_buffer_id0[5];
	struct composer_grapdata grap_data;
};


#endif
#endif
