/*
 *  File Name       : RSPF0_FMSIDRV_API.h
 *  Function        : SPU OPEN SOUND I/F
 *  Release Version : Ver 1.00
 *  Release Date    : 2011/11/30
 *
 *  Copyright (C) 2011 Renesas Electronics Corporation
 *
 *  This program is free software;you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by Free
 *  Softwere Foundation; either version 2 of License, or (at your option) any
 *  later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program;
 *  If not, write to the Free Software Foundation, Inc., 51 Franklin St,
 *  Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
#ifndef __RSPF0_FMSI_DRIVER_H__
#define __RSPF0_FMSI_DRIVER_H__

#include <linux/module.h>

#define RSPF0_FMSIDRV_RTN_SUCCESS               0
#define RSPF0_FMSIDRV_RTN_ERROR_PARAM          -1
#define RSPF0_FMSIDRV_RTN_ERROR_STATE          -2

enum {
	RSPF0_FMSIDRV_DIRECTION_SOUTPUT = 0,
	RSPF0_FMSIDRV_DIRECTION_SINPUT,
	RSPF0_FMSIDRV_DIRECTION_MAX
};

enum {
	RSPF0_FMSIDRV_BWS_24BIT = 0,
	RSPF0_FMSIDRV_BWS_16BIT,
	RSPF0_FMSIDRV_BWS_MAX
};

enum {
	RSPF0_FMSIDRV_FORMAT_MONO = 0,
	RSPF0_FMSIDRV_FORMAT_MONO_DELAY,
	RSPF0_FMSIDRV_FORMAT_PCM,
	RSPF0_FMSIDRV_FORMAT_I2S,
	RSPF0_FMSIDRV_FORMAT_TDM,
	RSPF0_FMSIDRV_FORMAT_TDM_DELAY,
	RSPF0_FMSIDRV_FORMAT_MAX
};

enum {
	RSPF0_FMSIDRV_TDM_CHANNEL_1 = 0,
	RSPF0_FMSIDRV_TDM_CHANNEL_2,
	RSPF0_FMSIDRV_TDM_CHANNEL_3,
	RSPF0_FMSIDRV_TDM_CHANNEL_4,
	RSPF0_FMSIDRV_TDM_CHANNEL_5,
	RSPF0_FMSIDRV_TDM_CHANNEL_6,
	RSPF0_FMSIDRV_TDM_CHANNEL_7,
	RSPF0_FMSIDRV_TDM_CHANNEL_8,
	RSPF0_FMSIDRV_TDM_CHANNEL_MAX
};

enum {
	RSPF0_FMSIDRV_MSMODE_SLAVE = 0,
	RSPF0_FMSIDRV_MSMODE_MASTER,
	RSPF0_FMSIDRV_MSMODE_MAX
};

enum {
	RSPF0_FMSIDRV_SEREALIN_DSP = 0,
	RSPF0_FMSIDRV_SEREALIN_MSU = 2,
	RSPF0_FMSIDRV_SEREALIN_MAX = 3
};

enum {
	RSPF0_FMSIDRV_SEREALOUT_DSP = 0,
	RSPF0_FMSIDRV_SEREALOUT_MSU,
	RSPF0_FMSIDRV_SEREALOUT_MAX
};

enum {
	RSPF0_FMSIDRV_AUDIOCLK_512FS = 0,
	RSPF0_FMSIDRV_AUDIOCLK_256FS,
	RSPF0_FMSIDRV_AUDIOCLK_128FS,
	RSPF0_FMSIDRV_AUDIOCLK_64FS,
	RSPF0_FMSIDRV_AUDIOCLK_32FS,
	RSPF0_FMSIDRV_AUDIOCLK_MAX
};

enum {
	RSPF0_FMSIDRV_BITPERFS_32BIT = 0,
	RSPF0_FMSIDRV_BITPERFS_64BIT,
	RSPF0_FMSIDRV_BITPERFS_128BIT,
	RSPF0_FMSIDRV_BITPERFS_256BIT,
	RSPF0_FMSIDRV_BITPERFS_512BIT,
	RSPF0_FMSIDRV_BITPERFS_16BIT = 7,
	RSPF0_FMSIDRV_BITPERFS_MAX
};

enum {
	RSPF0_FMSIDRV_POLARITY_NORMAL = 0,
	RSPF0_FMSIDRV_POLARITY_REVERSE,
	RSPF0_FMSIDRV_POLARITY_MAX
};

enum {
	RSPF0_FMSIDRV_MASTERCLOCK_TWO = 0,
	RSPF0_FMSIDRV_MASTERCLOCK_ONE,
	RSPF0_FMSIDRV_MASTERCLOCL_MAX
};

enum {
	RSPF0_FMSIDRV_SLAVECLOCK_TWO = 0,
	RSPF0_FMSIDRV_SLAVECLOCK_ONE,
	RSPF0_FMSIDRV_SLAVECLOCL_MAX
};

typedef struct {
	void (*read)(unsigned long *addr, unsigned long *data,
							unsigned long size);
	void (*write)(unsigned long *addr, unsigned long *data,
							unsigned long size);
	void (*wait)(unsigned long usec);
	unsigned long address_offset;
} RSPF0_FMSIDRV_InitParam;

typedef struct {
	unsigned long master_mode;
	unsigned long slave_mode;
} RSPF0_FMSIDRV_Clock;

typedef struct {
	unsigned long bws;
	unsigned long format;
	unsigned long tdm_channel;
	unsigned long msmode;
} RSPF0_FMSIDRV_IoParam;

typedef struct {
	unsigned long audio_clock_mode;
	unsigned long bit_per_fs_mode;
	struct {
		unsigned long lrclk_inverse;
		unsigned long bclk_inverse;
	} master;
	struct {
		unsigned long lrclk_inverse;
		unsigned long bclk_inverse;
	} slave;
} RSPF0_FMSIDRV_PortParam;

typedef struct RSPF0_FMSIDRV_Port_ RSPF0_FMSIDRV_Port;

#ifdef __cplusplus
extern "C" {
#endif
extern long RSPF0_FMSIDRV_Init(RSPF0_FMSIDRV_InitParam *param);
extern long RSPF0_FMSIDRV_OpenPort(RSPF0_FMSIDRV_Clock *clock,
			unsigned long direction, RSPF0_FMSIDRV_Port **port);
extern long RSPF0_FMSIDRV_ClosePort(RSPF0_FMSIDRV_Port *port);
extern long RSPF0_FMSIDRV_SetPortParam(RSPF0_FMSIDRV_Port *port,
						RSPF0_FMSIDRV_PortParam *param);
extern long RSPF0_FMSIDRV_SetIoParam(RSPF0_FMSIDRV_Port *port,
						RSPF0_FMSIDRV_IoParam *param);
extern long RSPF0_FMSIDRV_ResetPort(RSPF0_FMSIDRV_Port *port);
extern long RSPF0_FMSIDRV_StartPort(RSPF0_FMSIDRV_Port *port);
extern long RSPF0_FMSIDRV_SoftwareReset(void);
#ifdef __cplusplus
}
#endif
#endif /* __RSPF0_FMSI_DRIVER_H__ */

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("SPU OPEN SOUND I/F Ver 1.0");
MODULE_AUTHOR("Renesas Electronics");
